# -*- coding: utf-8 -*-
from __future__ import print_function, division, absolute_import

import functools
import os

from techblog_pyside.utils import common, qt
from techblog_pyside.mixin import menu

from PySide2 import QtWidgets, QtCore
from maya.app.general import mayaMixin

from techblog_pyside.ui import ui_form, ui_path


class PathForm(ui_path.Ui_Form):
    """UIの動的追加処理に設定したいUIの中で完結できる部分の処理を実装
    """
    def __init__(self):
        super(PathForm, self).__init__()
        self.__subject = common.Subject()

    @property
    def updated(self):
        """関数登録部分の外部公開
        """
        return self.__subject.listen

    def setupUi(self, Form):
        """SIGNAL・SLOTの追加設定
        """
        super(PathForm, self).setupUi(Form)
        self.lineEdit.textChanged.connect(self.__on_path_changed)
        self.toolButton.clicked.connect(functools.partial(self.__on_btn_clicked, self.lineEdit))

    def __on_path_changed(self):
        """LineEdit変更時の登録された関数への通知
        """
        self.__subject.emit(os.path.basename(self.lineEdit.text()))

    def __on_btn_clicked(self, line_edit):
        """ボタン押下時のLineEditへのフォルダパス設定
        """
        p = QtWidgets.QFileDialog.getExistingDirectory()
        if p:
            line_edit.setText(p)


class FrameLayoutWidget(mayaMixin.MayaQWidgetBaseMixin, qt.QtDefaultWidget):
    def __init__(self, parent=None):
        super(FrameLayoutWidget, self).__init__(parent)
        self.setLayout(QtWidgets.QVBoxLayout())
        self.setObjectName('FrameLayoutWidget')

        # PySideで作成したFrameLayout
        self.__dummy_frame = qt.FrameLayout(self)
        # FrameLayoutが閉じた状態になった時の処理を登録
        self.__dummy_frame.frame_btn.collapsed.add(lambda: print('collapsed'))
        # FrameLayoutが開いた状態になった時の処理を登録
        self.__dummy_frame.frame_btn.expanded.add(lambda: print('expanded'))
        # FrameLayoutをWidgetに追加
        self.layout().addWidget(self.__dummy_frame)

        for i in range(5):
            # FrameLayoutに追加するボタンを作成
            btn = QtWidgets.QPushButton(str(i))
            # ボタンが押されたときの処理を登録
            btn.clicked.connect(functools.partial(print, i))

            # FrameLayoutにボタンを追加
            self.__dummy_frame.frame_layout.addWidget(btn)


class QtVariadicWidget(mayaMixin.MayaQWidgetBaseMixin, qt.QtDefaultWidget):
    def __init__(self, parent=None):
        super(QtVariadicWidget, self).__init__(parent)
        self.setLayout(QtWidgets.QVBoxLayout())
        self.layout().setAlignment(QtCore.Qt.AlignTop)
        self.setObjectName('QtVariadicWidget')

        # 動的なUIの追加処理に使用するボタン
        btn_layout = QtWidgets.QHBoxLayout()
        # 追加用ボタン
        add_btn = QtWidgets.QPushButton('+', self)
        add_btn.setObjectName('add_btn')
        # 削除用ボタン
        sub_btn = QtWidgets.QPushButton('-', self)
        sub_btn.setObjectName('sub_btn')
        # Layoutに配置
        btn_layout.addWidget(add_btn)
        btn_layout.addWidget(sub_btn)
        self.layout().addLayout(btn_layout)

        # Layoutに対して動的なUI追加を行う処理の作成と追加できる最小・最大の設定
        self.__variadic = qt.QtVariadic(PathForm, self.layout(), 2, 10)
        # 追加用ボタンの登録
        self.__variadic.register_add_btn(add_btn)
        # 削除用ボタンの登録
        self.__variadic.register_sub_btn(sub_btn)
        # 動的に追加されたUIに対して設定するコールバック処理
        self.__variadic.register_callback({'updated.add': print})
        # 最小の初期化されたUIの作成
        self.__variadic.initialize()


class MixinWidget(mayaMixin.MayaQWidgetBaseMixin, qt.QtDefaultWidget):
    def __init__(self, parent=None):
        super(MixinWidget, self).__init__(parent)
        # サンプルUIを使用
        self.__ui = ui_form.Ui_Form()
        self.__ui.setupUi(self)
        self.setObjectName('MixinWidget')


class MenuMixinWidget(menu.MinimumFileMenuMixin, menu.MinimumHelpMenuMixin, menu.MenubarMixin, MixinWidget):
    def __init__(self, parent=None):
        # MixinWidget→MenubarMixin→MinimumHelpMenuMixin→MinimumFileMenuMixinの順でinitの処理が実行される
        super(MenuMixinWidget, self).__init__(parent)
        self.setObjectName('MenuMixinWidget')

        # メニュー作成
        self.create_file_menu()
        self.create_help_menu('https://www.monolithsoft.co.jp/')


class DummyTool(mayaMixin.MayaQWidgetBaseMixin, qt.QtDefaultWidget):
    """ダミーツール
    """
    def __init__(self, parent=None):
        super(DummyTool, self).__init__(parent)
        # サンプルUIを使用
        self.__ui = ui_form.Ui_Form()
        self.__ui.setupUi(self)
        self.setObjectName('DummyTool')

        # 動的なUIの追加処理に使用するボタン
        btn_layout = QtWidgets.QHBoxLayout()
        add_btn = QtWidgets.QPushButton('+', self)
        add_btn.setObjectName('add_btn')
        sub_btn = QtWidgets.QPushButton('-', self)
        sub_btn.setObjectName('sub_btn')
        btn_layout.addWidget(add_btn)
        btn_layout.addWidget(sub_btn)
        self.layout().addLayout(btn_layout)

        # PySideで作成したFrameLayout
        self.__dummy_frame = qt.FrameLayout(self)
        self.__dummy_frame.setObjectName('FrameLayout')
        self.__dummy_frame.frame_btn.collapsed.add(lambda: print('collapsed'))
        self.__dummy_frame.frame_btn.expanded.add(lambda: print('expanded'))
        self.layout().addWidget(self.__dummy_frame)

        # FrameLayoutに対して動的なUIの追加処理
        self.__variadic = qt.QtVariadic(PathForm, self.__dummy_frame.frame_layout, 2, 10)
        self.__variadic.register_add_btn(add_btn)
        self.__variadic.register_sub_btn(sub_btn)
        self.__variadic.register_callback({'updated.add': self.__on_result})
        self.__variadic.initialize()

        # 状態を保存したいUIにはオブジェクト名が必要
        self.__result_line = QtWidgets.QLineEdit(self)
        self.__result_line.setObjectName('result_line')
        self.layout().addWidget(self.__result_line)

    def __on_result(self, base_name):
        self.__result_line.setText(base_name)


class DummyMixinTool(menu.MinimumFileMenuMixin, menu.MinimumHelpMenuMixin, menu.MenubarMixin, DummyTool):
    """ダミーツールに各種Mixinを追加したツール
    """
    def __init__(self, parent=None):
        # DummyMixinTool→MenubarMixin→MinimumHelpMenuMixin→MinimumFileMenuMixin→CustomShelfMixinの順でinitの処理が実行される
        super(DummyMixinTool, self).__init__(parent)
        self.setObjectName('DummyMixinTool')

        # メニュー作成
        self.create_file_menu()
        self.create_help_menu('https://www.monolithsoft.co.jp/')

        # UIの状態を復元
        self.load_state()

    def closeEvent(self, *args, **kwargs):
        super(DummyMixinTool, self).closeEvent(*args, **kwargs)

        # UIの状態を保存
        self.save_state()


def frame_main():
    """FrameLayoutサンプル
    """
    win = FrameLayoutWidget()
    win.show()

    return win


def variadic_main():
    """UI動的追加サンプル
    """
    win = QtVariadicWidget()
    win.show()

    return win


def default_main():
    """仮ツールサンプル
    """
    win = MixinWidget()
    win.show()

    return win


def menu_main():
    """仮ツールにメニュー機能追加サンプル
    """
    win = MenuMixinWidget()
    win.show()

    return win


def main():
    """全部乗せサンプル
    """
    win = DummyMixinTool()
    win.show()

    return win
