# -*- coding: utf-8 -*-
from __future__ import print_function, division, absolute_import

from PySide2 import QtWidgets, QtGui, QtCore
from . import common


class QtDefaultWidget(QtWidgets.QWidget):
    """QWidgetの自動削除Attribute設定
    """
    def __init__(self, parent=None, *args, **kwargs):
        super(QtDefaultWidget, self).__init__(parent=parent, *args, **kwargs)
        self.setAttribute(QtCore.Qt.WA_DeleteOnClose)


class QtVariadic(QtCore.QObject):
    """動的にUIを追加するクラス
    """
    def __init__(self, widget_cls, layout, num_min=0, num_max=999, parent=None):
        super(QtVariadic, self).__init__(parent)
        assert num_min < num_max

        self.__widget_cls = widget_cls
        self.__layout = layout
        self.__instanced = []
        self.__btns = []

        self.__callback_dict = {}

        self.__min = num_min
        self.__max = num_max

    @property
    def widgets(self):
        """動的に追加されたUIのQWidget
        """
        return [item[0] for item in self.__instanced]

    @property
    def instances(self):
        """動的に追加されたUIのクラスオブジェクト
        """
        return [item[1] for item in self.__instanced]

    def initialize(self):
        """設定した最低個数の状態に更新
        """
        for w in self.__instanced:
            self.__sub_process()

        for i in range(self.__min):
            self.__add_process()

    def register_add_btn(self, btn):
        """動的にUIを追加するためのボタン登録
        """
        if btn in self.__btns:
            return

        self.__btns.append(btn)
        btn.clicked.connect(self.__add_process)
        pass

    def register_sub_btn(self, btn):
        """動的にUIを削除するためのボタン登録
        """
        if btn in self.__btns:
            return

        self.__btns.append(btn)
        btn.clicked.connect(self.__sub_process)
        pass

    def register_callback(self, callback_dict):
        """動的にUIが追加されるたびに設定するコールバック関数の登録
        """
        self.__callback_dict = callback_dict

    def __add_process(self):
        """動的追加内部処理
        """
        if not len(self.__instanced) < self.__max:
            return

        w = QtWidgets.QWidget()
        w_var = self.__widget_cls()
        w_var.setupUi(w)

        # 重複がないであろうオブジェクト名の設定
        w.setObjectName(w.objectName() + '_' + str(len(self.__instanced)) + '_variadic_')
        for c in w.children():
            c.setObjectName(c.objectName() + '_' + str(len(self.__instanced)) + '_variadic_')

        self.__callback_process(w_var)

        self.__layout.addWidget(w)
        self.__instanced.append((w, w_var))

    def __sub_process(self):
        """動的削除内部処理
        """
        if not len(self.__instanced) > self.__min:
            return

        if not any(self.__instanced):
            return

        w, _ = self.__instanced.pop(-1)
        self.__layout.removeWidget(w)
        w.deleteLater()

    def __callback_process(self, w_var):
        """関数登録内部処理
        """
        for attr, cb in self.__callback_dict.items():
            slot_func = common.get_attr_chain(w_var, attr)

            if not slot_func:
                continue
            slot_func(cb)


class _FrameButton(QtWidgets.QWidget):
    """FrameLayoutのボタン部分

    FrameLayoutが開いている・閉じている状態はボタン側で持つ
    """
    def __init__(self, label='', parent=None):
        super(_FrameButton, self).__init__(parent)
        self.__close_pix = QtGui.QPixmap(':/teRightArrow.png')
        self.__open_pix = QtGui.QPixmap(':/teDownArrow.png')

        self.setMinimumSize(100, 20)
        self.setSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        self.setAutoFillBackground(True)

        p = self.palette()
        p.setColor(self.backgroundRole(), QtGui.QColor(93, 93, 93))
        self.setPalette(p)

        layout = QtWidgets.QHBoxLayout()
        layout.setContentsMargins(5, 0, 5, 0)
        layout.setAlignment(QtCore.Qt.AlignLeft | QtCore.Qt.AlignVCenter)
        self.setLayout(layout)

        self.__icon = QtWidgets.QLabel()
        self.__icon.setPixmap(self.__close_pix)
        layout.addWidget(self.__icon)
        self.__label = QtWidgets.QLabel(unicode(' ' + label))
        layout.addWidget(self.__label)

        self.__toggle = False
        self.__collapsed = common.Subject()
        self.__expanded = common.Subject()
        self.__collapsed.listen.add(lambda: self.__icon.setPixmap(self.__close_pix))
        self.__expanded.listen.add(lambda: self.__icon.setPixmap(self.__open_pix))

    @property
    def toggle(self):
        """プログラムによる状態取得用処理
        """
        return self.__toggle

    @toggle.setter
    def toggle(self, value):
        """プログラムによる状態設定用処理
        """
        self.__toggle = value
        self.__expanded.emit() if self.__toggle else self.__collapsed.emit()

    @property
    def collapsed(self):
        """FrameLayoutが閉じられた際の関数登録
        """
        return self.__collapsed.listen

    @property
    def expanded(self):
        """FrameLayoutが開かれた際の関数登録
        """
        return self.__expanded.listen

    def setFont(self, font):
        """Font設定
        """
        self.__label.setFont(font)

    def mouseReleaseEvent(self, *args, **kwargs):
        """FrameLayoutボタンのクリック判定
        """
        self.__toggle = not self.__toggle
        self.__expanded.emit() if self.__toggle else self.__collapsed.emit()


class FrameLayout(QtWidgets.QWidget):
    """PySideで作成したFrameLayout
    """
    def __init__(self, parent=None):
        super(FrameLayout, self).__init__(parent)

        self.setSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        layout = QtWidgets.QVBoxLayout()
        layout.setContentsMargins(0, 0, 0, 0)
        layout.setAlignment(QtCore.Qt.AlignTop)
        self.setLayout(layout)

        self.__frame_btn = _FrameButton('Option')
        self.__frame_btn.setObjectName('frame_btn')
        self.__frame_btn.collapsed.add(self.__on_collapsed)
        self.__frame_btn.expanded.add(self.__on_expanded)

        self.__frame = QtWidgets.QWidget(self)
        self.__frame.setObjectName('frame_widget')
        self.__frame_layout = QtWidgets.QVBoxLayout(self.__frame)
        self.__frame_layout.setObjectName('frame_layout')
        self.__frame_layout.setMargin(0)

        self.layout().addWidget(self.__frame_btn)
        self.layout().addWidget(self.__frame)

        self.__frame_btn.toggle = True

    @property
    def frame_btn(self):
        """_FrameButtonへのアクセス
        """
        return self.__frame_btn

    @property
    def frame_layout(self):
        """Layout部分へのアクセス
        """
        return self.__frame_layout

    def __on_collapsed(self):
        """_FrameButtonが閉じられた状態になった時にLayout内のItemを非表示
        """
        for item in self.__frame.findChildren(QtWidgets.QWidget):
            item.setVisible(False)

    def __on_expanded(self):
        """_FrameButtonが開いた状態になった時にLayout内のItemを表示
        """
        for item in self.__frame.findChildren(QtWidgets.QWidget):
            item.setVisible(True)
